from sklearn.base import BaseEstimator
from lifetimes import BetaGeoFitter, GammaGammaFitter
import pandas as pd


class LifetimesCLVModel(BaseEstimator):
    """Lifetimes CLV prediction"""

    def __init__(
        self,
        frequency_col="frequency",
        t_col="customer_age",
        recency_col="recency",
        monetary_col="monetary_value",
        column_labels=None,
        penalizer_bgf=0.5,
        penilizer_ggf=0.0,
        forward_window=12,
        discount_rate=0.01,
        lifetime_activation=True,
    ):
        self.frequency_col = frequency_col
        self.t_col = t_col
        self.recency_col = recency_col
        self.monetary_col = monetary_col

        self.penalizer_bgf = penalizer_bgf
        self.penilizer_ggf = penilizer_ggf

        self.column_labels = column_labels
        self.bgf = BetaGeoFitter(penalizer_coef=self.penalizer_bgf)
        self.ggf = GammaGammaFitter(penalizer_coef=self.penilizer_ggf)
        self.forward_window = forward_window  # month
        self.discount_rate = discount_rate  # monthly discount rate ~ 12.7% annually
        self.lifetime_activation = lifetime_activation

    def get_relevant_columns(self, X, columns_name):
        # Retrieve the index of the important column
        column_index = self.column_labels.index(columns_name)
        # Retrieve the corresponding data column
        return X[:, column_index]

    def build_input_df(self, X):
        return pd.DataFrame(
            {
                "frequency": self.get_relevant_columns(X, self.frequency_col),
                "customer_age": self.get_relevant_columns(X, self.t_col),
                "recency": self.get_relevant_columns(X, self.recency_col),
                "monetary": self.get_relevant_columns(X, self.monetary_col),
            },
            dtype="float"
        )

    def fit(self, X, y):
        if self.lifetime_activation:
            input_df = self.build_input_df(X)
            input_df = input_df[input_df["monetary"] > 0]
            returning_customers_summary = input_df[input_df["frequency"] > 0]

            self.bgf.fit(frequency=input_df["frequency"], T=input_df["customer_age"], recency=input_df["recency"])
            self.ggf.fit(
                frequency=returning_customers_summary["frequency"], monetary_value=returning_customers_summary["monetary"]
            )

    def set_column_labels(self, column_labels):
        # in order to preserve the attribute `column_labels` when cloning
        # the estimator, we have declared it as a keyword argument in the
        # `__init__` and set it there
        print(f"set_column_labels: {column_labels}")
        self.column_labels = column_labels

    def predict(self, X):
        input_df = self.build_input_df(X)
        if self.lifetime_activation:
            input_df_infer = input_df[input_df["monetary"] > 0]
            input_df["predicted_clv"] = self.ggf.customer_lifetime_value(
                self.bgf,  # the model to use to predict the number of future transactions
                frequency=input_df_infer["frequency"],
                recency=input_df_infer["recency"],
                T=input_df_infer["customer_age"],
                monetary_value=input_df_infer["monetary"],
                time=self.forward_window,  # months
                discount_rate=self.discount_rate,  # monthly discount rate ~ 12.7% annually
            )
            input_df["predicted_clv"] = input_df["predicted_clv"].fillna(0)
        else:
            input_df["predicted_clv"] = 0
        return input_df["predicted_clv"].values

    def fit_predict(self, X, y):
        self.fit(X, y)
        return self.predict(X)
